-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local GUILD_ROSTER_MAX_COLUMNS   = 5;
local GUILD_ROSTER_MAX_STRINGS   = 4;
local GUILD_ROSTER_BAR_MAX       = 239;
local GUILD_ROSTER_BUTTON_OFFSET = 2;
local GUILD_ROSTER_BUTTON_HEIGHT = 20;

local currentGuildView = view;

local filteredList = {};

local classFlag =
{
  ["DRUID"]       = 0x0001,
  ["DEATHKNIGHT"] = 0x0002,
  ["HUNTER"]      = 0x0004,
  ["MAGE"]        = 0x0008,
  ["PALADIN"]     = 0x0010,
  ["PRIEST"]      = 0x0020,
  ["ROGUE"]       = 0x0040,
  ["SHAMAN"]      = 0x0080,
  ["WARLOCK"]     = 0x0100,
  ["WARRIOR"]     = 0x0200
}

local additionalClasses =
{
  {name = L["Any"],     flags = 0xFFFF},
  {name = L["Healing"], flags = 0x00B1},
  {name = L["Tanking"], flags = 0x0213}
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

NigglesGuildRosterTabOrder =
{
  "NigglesGuildRosterName",
  "NigglesGuildRosterLevelMin",
  "NigglesGuildRosterLevelMax",
  "NigglesGuildRosterZone",
  "NigglesGuildRosterNote"
};

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to update the Niggles guild roster
--
-- NOTE: This is a modified version of 'GuildRoster_Update()' in
--       'Blizzard_GuildRoster.lua'.
--
function NigglesGuildRosterContainerUpdate()
  local scrollFrame = NigglesGuildRosterContainer;
  local offset = HybridScrollFrame_GetOffset(scrollFrame);
  local buttons = scrollFrame.buttons;
  local numButtons = #buttons;
  local button, index, class;
  local totalMembers, onlineMembers = GetNumGuildMembers();
  local selectedGuildMember = GetGuildRosterSelection();
  
  if (currentGuildView == "tradeskill") then
    return;
  end

  local guildName, guildRankName, guildRankIndex = GetGuildInfo("player");
  local maxRankIndex = GuildControlGetNumRanks() - 1;  
  local maxWeeklyXP, maxTotalXP = GetGuildRosterLargestContribution();
  local maxAchievementsPoints = GetGuildRosterLargestAchievementPoints();

  -- numVisible
  local visibleMembers = onlineMembers;
  if ( GetGuildRosterShowOffline() ) then
    visibleMembers = totalMembers;
  end
  visibleMembers = #filteredList;

  for i = 1, numButtons do
    button = buttons[i];    
    index = offset+i;
    
    if (index <= visibleMembers) then
      local name, rank, rankIndex, level, class, zone, note, officernote, online, isAway, classFileName, achievementPoints, achievementRank, isMobile, canSoR = GetGuildRosterInfo(filteredList[index]);

      button.guildIndex = filteredList[index];
      local displayedName = name;
      if ( isMobile ) then
        if (isAway == 2) then
          displayedName = MOBILE_BUSY_ICON..displayedName;
        elseif (isAway == 1) then
          displayedName = MOBILE_AWAY_ICON..displayedName;
        else
          displayedName = ChatFrame_GetMobileEmbeddedTexture(73/255, 177/255, 73/255)..displayedName;
        end
      end
      button.online = online;
      if ( currentGuildView == "playerStatus" ) then
        GuildRosterButton_SetStringText(button.string1, level, online)
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName)
        GuildRosterButton_SetStringText(button.string3, isMobile and REMOTE_CHAT or zone, online)
        
      elseif ( currentGuildView == "guildStatus" ) then
        GuildRosterButton_SetStringText(button.string1, displayedName, online, classFileName)
        GuildRosterButton_SetStringText(button.string2, rank, online)
        GuildRosterButton_SetStringText(button.string3, note, online)
        
        if ( online ) then
          GuildRosterButton_SetStringText(button.string4, GUILD_ONLINE_LABEL, online);          
        else
          GuildRosterButton_SetStringText(button.string4, GuildRoster_GetLastOnline(index), online);
        end
      elseif ( currentGuildView == "weeklyxp" ) then
        local weeklyXP, totalXP, weeklyRank, totalRank = GetGuildRosterContribution(index);
        GuildRosterButton_SetStringText(button.string1, level, online)
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName)
        GuildRosterButton_SetStringText(button.string3, weeklyXP, online)
        if ( weeklyXP == 0 ) then
          button.barTexture:Hide();
        else
          button.barTexture:SetWidth(weeklyXP / maxWeeklyXP * GUILD_ROSTER_BAR_MAX);
          button.barTexture:Show();
        end
        GuildRosterButton_SetStringText(button.barLabel, "#"..weeklyRank, online);
      elseif ( currentGuildView == "totalxp" ) then
        local weeklyXP, totalXP, weeklyRank, totalRank = GetGuildRosterContribution(index);
        GuildRosterButton_SetStringText(button.string1, level, online);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName);
        GuildRosterButton_SetStringText(button.string3, totalXP, online);
        if ( totalXP == 0 ) then
          button.barTexture:Hide();
        else
          button.barTexture:SetWidth(totalXP / maxTotalXP * GUILD_ROSTER_BAR_MAX);
          button.barTexture:Show();
        end
        GuildRosterButton_SetStringText(button.barLabel, "#"..totalRank, online);      
      elseif ( currentGuildView == "pve" ) then
        GuildRosterButton_SetStringText(button.string1, level, online);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName);
        GuildRosterButton_SetStringText(button.string3, valor, online);
        GuildRosterButton_SetStringText(button.string4, hero, online);
      elseif ( currentGuildView == "pvp" ) then
        local bgRating, arenaRating, arenaTeam = GetGuildRosterPVPRatings(index);
        GuildRosterButton_SetStringText(button.string1, level, online);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName);
        GuildRosterButton_SetStringText(button.string3, bgRating, online);
        GuildRosterButton_SetStringText(button.string4, string.format(GUILD_ROSTER_ARENA_RATING, arenaRating, arenaTeam, arenaTeam), online);
      elseif ( currentGuildView == "achievement" ) then
        GuildRosterButton_SetStringText(button.string1, level, online);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, online, classFileName);
        if ( achievementPoints >= 0 ) then
          GuildRosterButton_SetStringText(button.string3, achievementPoints, online);
          if ( achievementPoints == 0 ) then
            button.barTexture:Hide();
          else
            button.barTexture:SetWidth(achievementPoints / maxAchievementsPoints * GUILD_ROSTER_BAR_MAX);
            button.barTexture:Show();
          end
        else
          GuildRosterButton_SetStringText(button.string3, NO_ROSTER_ACHIEVEMENT_POINTS, online);
          button.barTexture:Hide();
        end
        GuildRosterButton_SetStringText(button.barLabel, "#"..achievementRank, online);
      end
      button:Show();
      if ( mod(index, 2) == 0 ) then
        button.stripe:SetTexCoord(0.36230469, 0.38183594, 0.95898438, 0.99804688);
      else
        button.stripe:SetTexCoord(0.51660156, 0.53613281, 0.88281250, 0.92187500);
      end
      if ( selectedGuildMember == index ) then
        button:LockHighlight();
      else
        button:UnlockHighlight();
      end
    else
      button:Hide();
    end
  end

  local totalHeight = visibleMembers * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
  local displayedHeight = numButtons * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
  HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);

  return;
end

--
-- Function called when the user selects a different guild roster view
--
function NigglesGuildRosterSetView(view)
  -- Local Variables
  local button;
  local numButtons = #NigglesGuildRosterContainer.buttons;
  local buttonFontString;
  local stdButton = GuildRosterContainer.buttons[1];
  local fontString;
  local stringInfo = {};
  local numPoints;
  local xOffset;
  local haveIcon = stdButton.icon:IsShown();
  local haveBar  = stdButton.barLabel:IsShown();

  -- Save the new view
  currentGuildView = view;

  -- Check if the new view is 'tradeskill'
  if (view == "tradeskill") then
    -- Hide the Niggles guild roster...
    NigglesGuildRosterShow(false);

    -- Disable the button for toggling the Niggles guild roster
    NigglesGuildRosterButton:Disable();
    return;
  end

  -- Get info about the first standard guild roster buttons
  for idx = 1, GUILD_ROSTER_MAX_STRINGS do
    -- Check if the font string is shown
    fontString = stdButton["string"..idx];
    if (fontString:IsShown()) then
      -- Get the font string's width and horizontal justification
      stringInfo[idx] =
      {
        width    = fontString:GetWidth(),
        justifyH = fontString:GetJustifyH(),
      }

      -- Find the X offset for the font string's left point
      numPoints = fontString:GetNumPoints();
      for pointIdx = 1, numPoints do
        point, _, _, xOffset, _ = fontString:GetPoint(pointIdx);
        if (point == "LEFT") then
          stringInfo[idx].leftOffset = xOffset;
          break;
        end
      end
    end
  end

  -- Adjust the Niggles guild roster buttons for the new view
  for idx = 1, numButtons do
    button = NigglesGuildRosterContainer.buttons[idx];
    for stringIdx = 1, GUILD_ROSTER_MAX_STRINGS do
      fontString = button["string"..stringIdx];
      if (stringInfo[stringIdx] ~= nil) then
        fontString:SetPoint("LEFT", stringInfo[stringIdx].leftOffset, 0);
        fontString:SetWidth(stringInfo[stringIdx].width);
        fontString:SetJustifyH(stringInfo[stringIdx].justifyH);
        fontString:Show();
      else
        fontString:Hide();
      end
    end

    -- Show/Hide the button icon and bar
    if (haveIcon) then
      button.icon:Show();
    else
      button.icon:Hide();
    end
    if (haveBar) then
      button.barLabel:Show();
      -- NOTE: The texture is shown in 'NigglesGuildRosterContainerUpdate()'.
    else
      button.barLabel:Hide();
      button.barTexture:Hide();
    end
    button.header:Hide();
  end
  
  -- Make sure the button for toggling the Niggles guild roster is enabled 
  NigglesGuildRosterButton:Enable();

  -- Update the Niggles guild roster
  NigglesGuildRosterContainerUpdate();

  return;
end

function NigglesGuildRosterInit()
  -- Local Variables
  local button;

  -- Create the button for toggling the Niggles guild roster
  button = CreateFrame("BUTTON", "NigglesGuildRosterButton", GuildRosterFrame,
    "NigglesGuildRosterButtonTemplate");
  button.icon:SetTexture("Interface\\Common\\UI-Searchbox-Icon");
  button.icon:SetTexCoord(0, 1, 0, 1);
  if (NigglesDBC.settings.UiGuildRosterFilter == true) then
    button:Show();
  else
    button:Hide();
  end

  -- Create the Niggles guild roster
  CreateFrame("FRAME", "NigglesGuildRoster", GuildRosterFrame,
    "NigglesGuildRosterTemplate");

  -- Initialise the Niggles guild roster scroll frame
  NigglesGuildRosterContainer.update = NigglesGuildRosterContainerUpdate;
  HybridScrollFrame_CreateButtons(NigglesGuildRosterContainer,
    "GuildRosterButtonTemplate", 0, 0, "TOPLEFT", "TOPLEFT", 0,
    -GUILD_ROSTER_BUTTON_OFFSET, "TOP", "BOTTOM");
  NigglesGuildRosterContainerScrollBar.doNotHide = true;

  -- Adjust the standard column headers 
  for idx = 1, GUILD_ROSTER_MAX_COLUMNS do
    -- Adjust the frame level of the column headers 
    button = _G["GuildRosterColumnButton"..idx];
    button:SetFrameLevel(button:GetFrameLevel()+1);

    -- Hook into the 'OnClick' event
    button:HookScript("OnClick", NigglesGuildRosterContainerUpdate);
  end

  -- Hook into functions that change the view
  hooksecurefunc("GuildRoster_SetView", NigglesGuildRosterSetView);

  -- Hook into frames that change the view
  GuildRosterShowOfflineButton:HookScript("OnClick",
    NigglesGuildRosterFilter);
  for idx = 1, #(NigglesGuildRosterContainer.buttons) do
    NigglesGuildRosterContainer.buttons[idx]:HookScript("OnClick",
      NigglesGuildRosterContainerUpdate);
  end
  
  -- Set the initial view for the Niggles guild roster
  NigglesGuildRosterSetView(GetCVar("guildRosterView"));

  -- Hide the Niggles guild roster
  NigglesGuildRosterShow(false);

  return;
end

--
-- Function to filter the Niggles guild roster in a sort time
--
function NigglesGuildRosterDelayedFilter()
  -- Create a timer to filter the Niggles guidl roster in a short time
  NigglesTimerCreate("GuildRosterFilter", NigglesGuildRosterFilter, nil, nil,
    1.5, false)
  return;
end

--
-- Function to filter the Niggles guild roster
--
function NigglesGuildRosterFilter()
  -- Local Variables
  local visibleMembers;
  local name;
  local rankIdx;
  local level;
  local class;
  local zone;
  local note;
  local classFileName;
  local filterValues = {};

  -- Work out how many guild members should be visible in the roster
  if (GetGuildRosterShowOffline()) then
    visibleMembers, _ = GetNumGuildMembers();
  else
    _, visibleMembers = GetNumGuildMembers();
  end

  -- Get the dropdown filters
  filterValues.class =
    UIDropDownMenu_GetSelectedValue(NigglesGuildRosterClass);
  filterValues.rank  =
    (tonumber(UIDropDownMenu_GetSelectedValue(NigglesGuildRosterRank)) or 0)-1;

  -- Get some of the edit boxs
  filterValues.name = NigglesGuildRosterName:GetText():lower();
  filterValues.zone = NigglesGuildRosterZone:GetText():lower();
  filterValues.note = NigglesGuildRosterNote:GetText():lower();

  -- Get the level range
  filterValues.levelMin = (tonumber(NigglesGuildRosterLevelMin:GetText()) or 0);
  filterValues.levelMax = (tonumber(NigglesGuildRosterLevelMax:GetText()) or 0);
  if (filterValues.levelMin > 0) and (filterValues.levelMax == 0) then
    filterValues.levelMax = filterValues.levelMin;
  elseif (filterValues.levelMin == 0) and (filterValues.levelMax > 0) then
    filterValues.levelMin = filterValues.levelMax;
  elseif (filterValues.levelMin > filterValues.levelMax) then
    filterValues.levelMin, filterValues.levelMax = 
      filterValues.levelMax, filterValues.levelMin;
  end

  -- Filter the guild members
  wipe(filteredList);
  for idx = 1, visibleMembers do
    -- Get the required info for the member
    name, _, rankIdx, level, _, zone, note, _, _, _, classFileName = 
      GetGuildRosterInfo(idx);

    -- Check if the member matches the filter
    if (((filterValues.name == nil) or
         ((name or ""):lower():find(filterValues.name) ~= nil)) and
        (bit.band(filterValues.class, classFlag[classFileName]) ~= 0) and
        ((filterValues.levelMin == 0) or 
        ((level >= filterValues.levelMin) and
         (level <= filterValues.levelMax))) and
        ((filterValues.rank < 0) or
         (rankIdx == filterValues.rank)) and
        ((filterValues.zone == nil) or
         ((zone or ""):lower():find(filterValues.zone) ~= nil)) and
        ((filterValues.note == nil) or
         ((note or ""):lower():find(filterValues.note) ~= nil))) then
      table.insert(filteredList, idx);
    end

  end

  NigglesGuildRosterContainerUpdate();

  return;
end

--
-- Function to
--
function NigglesGuildRosterClassOnClick(self, button)
  UIDropDownMenu_SetSelectedValue(NigglesGuildRosterClass, self.value);
  NigglesGuildRosterFilter();
  return;
end

--
-- Function to
--
function NigglesGuildRosterClassOnLoad(self)
  -- Initialise the dropdown
  UIDropDownMenu_Initialize(self, NigglesGuildRosterClass_OnShow);
  UIDropDownMenu_JustifyText(self, "LEFT");
  UIDropDownMenu_SetWidth(self, 100);
  UIDropDownMenu_SetSelectedValue(NigglesGuildRosterClass, 0xFFFF);

  return;
end

function NigglesGuildRosterClass_OnShow()
  -- Local Variables
  local info = UIDropDownMenu_CreateInfo();
  local selected = UIDropDownMenu_GetSelectedValue(NigglesGuildRosterClass);
  
  -- Initialise the dropdown info
  info.func = NigglesGuildRosterClassOnClick;
  
  -- Add the additional classes to the dropdown
  for _, classInfo in ipairs(additionalClasses) do
    info.value     = classInfo.flags;
    info.text      = classInfo.name;
    info.checked   = (selected == classInfo.flags);
    UIDropDownMenu_AddButton(info);
  end

  -- Add the classes to the dropdown
  for class, localizedName in pairs(LOCALIZED_CLASS_NAMES_MALE) do
    info.value     = classFlag[class];
    info.text      = localizedName;
    info.checked   = (class == selected);
    info.colorCode = string.format("|cFF%02x%02x%02x",
      (255*RAID_CLASS_COLORS[class].r),
      (255*RAID_CLASS_COLORS[class].g),
      (255*RAID_CLASS_COLORS[class].b));
    UIDropDownMenu_AddButton(info);
  end

  return;
end

function NigglesGuildRosterRankOnClick(self, button)
  UIDropDownMenu_SetSelectedValue(NigglesGuildRosterRank, self.value);
  NigglesGuildRosterFilter();
  return;
end

function NigglesGuildRosterRankOnLoad(self)
  -- Initialise the dropdown
  UIDropDownMenu_Initialize(self, NigglesGuildRosterRank_OnShow);
  UIDropDownMenu_JustifyText(self, "LEFT");
  UIDropDownMenu_SetWidth(self, 100);
  UIDropDownMenu_SetSelectedValue(NigglesGuildRosterRank, 0);

  return;
end

function NigglesGuildRosterRank_OnShow()
  -- Local Variables
  local info = UIDropDownMenu_CreateInfo();
  local numRanks = GuildControlGetNumRanks();
  local selected = UIDropDownMenu_GetSelectedValue(NigglesGuildRosterRank);
  
  -- Initialise the dropdown info
  info.func = NigglesGuildRosterRankOnClick;
  
  -- Add 'Any' to the dropdown
  info.text    = L["Any"];
  info.value   = 0;
  info.checked = (idx == selected);
  UIDropDownMenu_AddButton(info);

  -- Add the guild's ranks to the dropdown
  for idx = 1, numRanks do
    info.text    = GuildControlGetRankName(idx);
    info.value   = idx;
    info.checked = (idx == selected);
    UIDropDownMenu_AddButton(info);
  end

  return;
end

--
-- Function for processing clicks on the Niggles guild roster toggle
--
function NigglesGuildRosterButtonClicked(self, button)
  -- Show/Hide the Niggles Guild Roster
  if (currentGuildView ~= "tradeskill") then
    NigglesGuildRosterShow();
    PlaySound("igMainMenuOptionCheckBoxOn");
  end

  return;
end

--
-- Function to show/hide the Niggles guild roster, with its additional filters
--
function NigglesGuildRosterShow(show)
  -- Clear any selection
  SetGuildRosterSelection(0);
  GuildFrame.selectedGuildMember = 0;
  GuildMemberDetailFrame:Hide();

  -- Work out if the Niggles guild roster should be shown
  if (show == nil) then
    show = not NigglesGuildRoster:IsShown();
  end

  -- Show/Hide the Niggles guild roster
  if (show) then
    -- Set all the filters to their default values
    NigglesGuildRosterName:SetText("");
    UIDropDownMenu_Initialize(NigglesGuildRosterClass,
      NigglesGuildRosterClass_OnShow);
    UIDropDownMenu_SetSelectedValue(NigglesGuildRosterClass, 0xFFFF);
    NigglesGuildRosterZone:SetText("");
    NigglesGuildRosterLevelMin:SetText("");
    NigglesGuildRosterLevelMax:SetText("");
    UIDropDownMenu_Initialize(NigglesGuildRosterRank,
      NigglesGuildRosterRank_OnShow);
    UIDropDownMenu_SetSelectedValue(NigglesGuildRosterRank, 0);
    NigglesGuildRosterNote:SetText("");

    -- Initialise the filtered list
    NigglesGuildRosterFilter();

    -- Show the Niggles guild roster
    NigglesGuildRoster:Show();

    -- Hide the standard guild roster
    GuildRosterContainer:Hide();

    -- Move the column buttons to the top of the Niggles guild roster
    GuildRosterColumnButton1:ClearAllPoints();
    GuildRosterColumnButton1:SetPoint("TOPLEFT", GuildRosterFrame,
      "TOPLEFT", 7, -150);
  else
    -- Hide the Niggles Guild Roster
    NigglesGuildRoster:Hide();

    -- Show the standard guild roster
    GuildRosterContainer:Show();
   
    -- Move the column buttons to the top of the standard guild roster
    GuildRosterColumnButton1:ClearAllPoints();
    GuildRosterColumnButton1:SetPoint("TOPLEFT", GuildRosterFrame,
      "TOPLEFT", 7, -68);

    -- Force the standard guild roster to update
    GuildRoster_Update();
  end

  return;
end
