-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local dropDownMenuInfo =
{
  LootMethod =
  {
    {
      label = L["FreeForAll"],
      value = "freeforall"
    },
    {
      label = L["RoundRobin"],
      value = "roundrobin"
    },
    {
      label = L["MasterLooter"],
      value = "master"
    },
    {
      label = L["GroupLoot"],
      value = "group"
    },
    {
      label = L["NeedGreed"],
      value = "needbeforegreed"
    }
  },
  LootThreshold =
  {
    {
      label = ITEM_QUALITY2_DESC,
      value = 2,
      color = ITEM_QUALITY_COLORS[2].hex
    },
    {
      label = ITEM_QUALITY3_DESC,
      value = 3,
      color = ITEM_QUALITY_COLORS[3].hex
    },
    {
      label = ITEM_QUALITY4_DESC,
      value = 4,
      color = ITEM_QUALITY_COLORS[4].hex
    }
  }
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable frames in the Niggles Options Raids panel
--
function NigglesOptionsRaidsEnableFrames()
  -- Enable or disable frames based on the updated options
  if (Niggles.updatedOptions.settings.RaidsAutoLootMethod == true) then
    UIDropDownMenu_EnableDropDown(NigglesOptionsRaidsLootMethod);
  else
    UIDropDownMenu_DisableDropDown(NigglesOptionsRaidsLootMethod);
  end
  if (Niggles.updatedOptions.settings.RaidsAutoLootThold == true) then
    UIDropDownMenu_EnableDropDown(NigglesOptionsRaidsLootThold);
  else
    UIDropDownMenu_DisableDropDown(NigglesOptionsRaidsLootThold);
  end
  NigglesEnableFrame(NigglesOptionsRaidsAssists,
    Niggles.updatedOptions.settings.RaidsAutoPromote);

  return;
end

--
-- Function to initialise a raid loot menu
--
function NigglesOptionsRaidsLootMenu_OnLoad(dropDown, setting, menuName, width)
  -- Initialise some variables for the dropDown menu
  dropDown.type          = CONTROLTYPE_DROPDOWN;
  dropDown.SetValue      = NigglesOptionsDropDown_SetValue;
  dropDown.nglSetting    = setting;
  dropDown.nglMenuItems  = dropDownMenuInfo[menuName];
  dropDown.nglObjectType = "UIDropDownMenu";

  -- Initialise the dropDown menu
  UIDropDownMenu_Initialize(dropDown, NigglesOptionsRaidsLootMenu_Initialise);
  UIDropDownMenu_JustifyText(dropDown, "LEFT");
  UIDropDownMenu_SetWidth(dropDown, width);

  return;
end

--
-- Function to initialise a drop down menu for the Niggles Options Raids panel
--
function NigglesOptionsRaidsLootMenu_Initialise(dropDown)
  -- Local Variables
  local selectedValue = UIDropDownMenu_GetSelectedValue(dropDown);
  local info          = UIDropDownMenu_CreateInfo();

  -- Check the required variables have been set
  if (dropDown.nglMenuItems) then
    -- Add each menu item in the supplied table
    for _, element in pairs(dropDown.nglMenuItems) do
      info.text        = L[element.label];
      info.func        = NigglesOptionsDropDownOnClick;
      info.arg1        = dropDown;
      info.value       = element.value;
      info.colorCode   = element.color;
      info.tooltipText = element.tooltip;
      if (info.value == selectedValue) then
        info.checked = 1;
      else
        info.checked = nil;
      end
      UIDropDownMenu_AddButton(info);
    end
  end

  return;
end

--
-- Function to refresh values in the Niggles Options Calendar
-- Events panel
--
function NigglesOptionsRefreshRaids()
  -- Local Variables
  local widget;

  -- Initialise and update the raid assistants list
  widget      = NigglesOptionsRaidsAssists;
  widget.list = Niggles.updatedOptions.raidAssists;
  FauxScrollFrame_OnVerticalScroll(_G[widget:GetName().."Scrollbar"],
    0, 14, NigglesCharacterListUpdate);

  -- Initialise all the widgets to the correct state for the settings
  NigglesOptionsRefreshSettings(NigglesOptionsRaids);

  -- Initialise all the frames to the correct state for the settings
  NigglesOptionsRaidsEnableFrames();

  return;
end

--
-- Function to promote raid assistants
--
function NigglesRaidRosterPromoteAssists()
  -- Local Variables
  local numMembers = GetRealNumRaidMembers();
  local name;
  local rank;

  -- Check if there are any raid assistants
  if (next(NigglesDB.raidAssists) ~= nil) then
    -- Process all the raid members
    for idx = 1, numMembers do
      name, rank = GetRaidRosterInfo(idx);
      if ((name ~= nil) and
          (rank == 0) and
          (NigglesTableContains(NigglesDB.raidAssists, name))) then
        PromoteToAssistant(name, true);
      end
    end
  end

  return;
end
